@@ -32,10 +32,25 @@ static const Feature self_advertised_features[] =
   { VERSION, NS_GOOGLE_TRANSPORT_P2P, PRESENCE_CAP_GOOGLE_TRANSPORT_P2P},
   { VERSION, NS_JINGLE, PRESENCE_CAP_JINGLE},
   { VERSION, NS_CHAT_STATES, PRESENCE_CAP_CHAT_STATES},
+  { VERSION, NS_NICK, 0},
+  { VERSION, NS_NICK "+notify", 0},
+  { VERSION, NS_OLPC_BUDDY_PROPS, 0},
+  { VERSION, NS_OLPC_BUDDY_PROPS "+notify", 0},
+  { VERSION, NS_OLPC_ACTIVITIES, 0},
+  { VERSION, NS_OLPC_ACTIVITIES "+notify", 0},
+  { VERSION, NS_OLPC_CURRENT_ACTIVITY, 0},
+  { VERSION, NS_OLPC_CURRENT_ACTIVITY "+notify", 0},
+  { VERSION, NS_OLPC_ACTIVITY_PROPS, 0},
+  { VERSION, NS_OLPC_ACTIVITY_PROPS "+notify", 0},
+  { VERSION, NS_SI, PRESENCE_CAP_SI},
+  { VERSION, NS_IBB, PRESENCE_CAP_IBB},
+  { VERSION, NS_TUBES, PRESENCE_CAP_SI_TUBES},
 
   { BUNDLE_VOICE_V1, NS_GOOGLE_FEAT_VOICE, PRESENCE_CAP_GOOGLE_VOICE},
-  { BUNDLE_JINGLE_AUDIO, NS_JINGLE_DESCRIPTION_AUDIO, PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO},
-  { BUNDLE_JINGLE_VIDEO, NS_JINGLE_DESCRIPTION_VIDEO, PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO},
+  { BUNDLE_JINGLE_AUDIO, NS_JINGLE_DESCRIPTION_AUDIO,
+    PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO},
+  { BUNDLE_JINGLE_VIDEO, NS_JINGLE_DESCRIPTION_VIDEO,
+    PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO},
   { NULL, NULL, 0}
 };
 
@@ -78,8 +93,11 @@ capabilities_get_initial_caps ()
   for (feat = self_advertised_features; NULL != feat->ns; feat++)
     {
       if (g_str_equal (feat->bundle, VERSION))
-          /* VERSION == bundle means a fixed feature, which we always advertise */
-            ret |= feat->caps;
+        {
+          /* VERSION == bundle means a fixed feature, which we always
+           * advertise */
+          ret |= feat->caps;
+        }
     }
 
   return ret;
@@ -87,7 +105,9 @@ capabilities_get_initial_caps ()
 
 const CapabilityConversionData capabilities_conversions[] =
 {
-  { TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, _gabble_media_channel_typeflags_to_caps, _gabble_media_channel_caps_to_typeflags },
+  { TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
+    _gabble_media_channel_typeflags_to_caps,
+    _gabble_media_channel_caps_to_typeflags },
   { NULL, NULL, NULL}
 };
 
