@@ -22,31 +22,29 @@
 
 #include "gabble-connection.h"
 
-#define DBUS_API_SUBJECT_TO_CHANGE
-
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
-#include <glib-object.h>
-#include <loudmouth/loudmouth.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/enums.h>
-#include <telepathy-glib/errors.h>
-#include <telepathy-glib/interfaces.h>
+#define DBUS_API_SUBJECT_TO_CHANGE
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+#include <glib-object.h>
+#include <loudmouth/loudmouth.h>
 
 #include <telepathy-glib/channel-iface.h>
 #include <telepathy-glib/channel-factory-iface.h>
-
+#include <telepathy-glib/dbus.h>
+#include <telepathy-glib/enums.h>
+#include <telepathy-glib/errors.h>
 #include <telepathy-glib/handle-repo-dynamic.h>
 #include <telepathy-glib/handle-repo-static.h>
-
+#include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/svc-connection.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_CONNECTION
 
+#include "bytestream-factory.h"
 #include "capabilities.h"
 #include "conn-aliasing.h"
 #include "conn-avatars.h"
@@ -54,24 +52,22 @@
 #include "conn-olpc.h"
 #include "debug.h"
 #include "disco.h"
-#include "presence-cache.h"
-#include "presence.h"
+#include "gabble-media-channel.h"
 #include "gabble-register.h"
+#include "gabble-roomlist-channel.h"
 #include "im-factory.h"
 #include "media-factory.h"
 #include "muc-factory.h"
 #include "namespaces.h"
+#include "presence-cache.h"
+#include "presence.h"
 #include "pubsub.h"
 #include "request-pipeline.h"
 #include "roster.h"
+#include "private-tubes-factory.h"
 #include "util.h"
 #include "vcard-manager.h"
 
-#include "gabble-media-channel.h"
-#include "gabble-roomlist-channel.h"
-
-#define GABBLE_TP_ALIAS_PAIR_TYPE (dbus_g_type_get_struct ("GValueArray", \
-      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID))
 #define GABBLE_TP_CAPABILITY_PAIR_TYPE (dbus_g_type_get_struct ("GValueArray", \
       G_TYPE_STRING, G_TYPE_UINT, G_TYPE_INVALID))
 #define GABBLE_TP_CAPABILITIES_CHANGED_MONSTER_TYPE (dbus_g_type_get_struct \
@@ -109,6 +105,7 @@ enum
     PROP_CONNECT_SERVER = 1,
     PROP_PORT,
     PROP_OLD_SSL,
+    PROP_REQUIRE_ENCRYPTION,
     PROP_REGISTER,
     PROP_LOW_BANDWIDTH,
     PROP_STREAM_SERVER,
@@ -143,6 +140,7 @@ struct _GabbleConnectionPrivate
   gchar *connect_server;
   guint port;
   gboolean old_ssl;
+  gboolean require_encryption;
 
   gboolean ignore_ssl_errors;
   TpConnectionStatusReason ssl_error;
@@ -182,7 +180,6 @@ struct _GabbleConnectionPrivate
 #define GABBLE_CONNECTION_GET_PRIVATE(obj) \
     ((GabbleConnectionPrivate *)obj->priv)
 
-static void connection_nickname_update_cb (GObject *, TpHandle, gpointer);
 static void connection_capabilities_update_cb (GabblePresenceCache *,
     TpHandle, GabblePresenceCapabilities, GabblePresenceCapabilities,
     gpointer);
@@ -196,14 +193,14 @@ _gabble_connection_create_channel_factories (TpBaseConnection *conn)
 
   self->roster = gabble_roster_new (self);
   g_signal_connect (self->roster, "nickname-update", G_CALLBACK
-      (connection_nickname_update_cb), self);
+      (gabble_conn_aliasing_nickname_updated), self);
 
   g_ptr_array_add (channel_factories, self->roster);
 
-  g_ptr_array_add (channel_factories,
-                   g_object_new (GABBLE_TYPE_MUC_FACTORY,
-                                 "connection", self,
-                                 NULL));
+  self->muc_factory = g_object_new (GABBLE_TYPE_MUC_FACTORY,
+      "connection", self,
+      NULL);
+  g_ptr_array_add (channel_factories, self->muc_factory);
 
   g_ptr_array_add (channel_factories,
                    g_object_new (GABBLE_TYPE_MEDIA_FACTORY,
@@ -215,6 +212,9 @@ _gabble_connection_create_channel_factories (TpBaseConnection *conn)
                                  "connection", self,
                                  NULL));
 
+  self->private_tubes_factory = gabble_private_tubes_factory_new (self);
+  g_ptr_array_add (channel_factories, self->private_tubes_factory);
+
   return channel_factories;
 }
 
@@ -233,11 +233,11 @@ gabble_connection_constructor (GType type,
   self->disco = gabble_disco_new (self);
   self->vcard_manager = gabble_vcard_manager_new (self);
   g_signal_connect (self->vcard_manager, "nickname-update", G_CALLBACK
-      (connection_nickname_update_cb), self);
+      (gabble_conn_aliasing_nickname_updated), self);
 
   self->presence_cache = gabble_presence_cache_new (self);
   g_signal_connect (self->presence_cache, "nickname-update", G_CALLBACK
-      (connection_nickname_update_cb), self);
+      (gabble_conn_aliasing_nickname_updated), self);
   g_signal_connect (self->presence_cache, "capabilities-update", G_CALLBACK
       (connection_capabilities_update_cb), self);
 
@@ -247,6 +247,8 @@ gabble_connection_constructor (GType type,
   conn_presence_init (self);
   conn_olpc_activity_properties_init (self);
 
+  self->bytestream_factory = gabble_bytestream_factory_new (self);
+
   return (GObject *)self;
 }
 
@@ -289,6 +291,9 @@ gabble_connection_get_property (GObject    *object,
     case PROP_OLD_SSL:
       g_value_set_boolean (value, priv->old_ssl);
       break;
+    case PROP_REQUIRE_ENCRYPTION:
+      g_value_set_boolean (value, priv->require_encryption);
+      break;
     case PROP_REGISTER:
       g_value_set_boolean (value, priv->do_register);
       break;
@@ -360,6 +365,9 @@ gabble_connection_set_property (GObject      *object,
     case PROP_OLD_SSL:
       priv->old_ssl = g_value_get_boolean (value);
       break;
+    case PROP_REQUIRE_ENCRYPTION:
+      priv->require_encryption = g_value_get_boolean (value);
+      break;
     case PROP_REGISTER:
       priv->do_register = g_value_get_boolean (value);
       break;
@@ -545,6 +553,14 @@ gabble_connection_class_init (GabbleConnectionClass *gabble_connection_class)
                                      G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_OLD_SSL, param_spec);
 
+  param_spec = g_param_spec_boolean ("require-encryption", "Require encryption",
+                                     "Require the connection to be encrypted, either "
+                                     "via old-style SSL, or StartTLS mechanisms.", FALSE,
+                                     G_PARAM_READWRITE |
+                                     G_PARAM_STATIC_NAME |
+                                     G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_REQUIRE_ENCRYPTION, param_spec);
+
   param_spec = g_param_spec_boolean ("register", "Register account on server",
                                      "Register a new account on server.", FALSE,
                                      G_PARAM_READWRITE |
@@ -717,11 +733,8 @@ gabble_connection_dispose (GObject *object)
             (base->status == TP_INTERNAL_CONNECTION_STATUS_NEW));
   g_assert (base->self_handle == 0);
 
-  g_object_unref (self->vcard_manager);
-  self->vcard_manager = NULL;
-
-  /* unreffing channel factories frees the roster */
-  self->roster = NULL;
+  g_object_unref (self->bytestream_factory);
+  self->bytestream_factory = NULL;
 
   g_object_unref (self->disco);
   self->disco = NULL;
@@ -729,6 +742,15 @@ gabble_connection_dispose (GObject *object)
   g_object_unref (self->req_pipeline);
   self->req_pipeline = NULL;
 
+  g_object_unref (self->vcard_manager);
+  self->vcard_manager = NULL;
+
+  /* remove borrowed references before TpBaseConnection unrefs the channel
+   * factories */
+  self->roster = NULL;
+  self->muc_factory = NULL;
+  self->private_tubes_factory = NULL;
+
   if (self->self_presence != NULL)
     g_object_unref (self->self_presence);
   self->self_presence = NULL;
@@ -737,7 +759,8 @@ gabble_connection_dispose (GObject *object)
   self->presence_cache = NULL;
 
   g_hash_table_destroy (self->olpc_activities_info);
-  g_hash_table_destroy (self->olpc_contacts_activities);
+  g_hash_table_destroy (self->olpc_pep_activities);
+  g_hash_table_destroy (self->olpc_invited_activities);
 
   /* if this is not already the case, we'll crash anyway */
   g_assert (!lm_connection_is_open (self->lmconn));
@@ -902,7 +925,7 @@ message_send_reply_cb (LmMessageHandler *handler,
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
-  if (handler_data->object_alive)
+  if (handler_data->object_alive && handler_data->reply_func != NULL)
     {
       return handler_data->reply_func (handler_data->conn,
                                        handler_data->sent_msg,
@@ -948,6 +971,9 @@ message_send_handler_destroy_cb (gpointer data)
  *
  * If object is non-NULL the handler will follow the lifetime of that object,
  * which means that if the object is destroyed the callback will not be invoked.
+ *
+ * if reply_func is NULL the reply will be ignored but connection_iq_unknown_cb
+ * won't be called.
  */
 gboolean
 _gabble_connection_send_with_reply (GabbleConnection *conn,
@@ -1151,6 +1177,7 @@ _gabble_connection_connect (TpBaseConnection *base,
       lm_connection_set_server (conn->lmconn, priv->connect_server);
       lm_connection_set_port (conn->lmconn, priv->port);
     }
+#ifndef HAVE_LM_SRV_LOOKUPS
   /* otherwise set the server & port to the stream server,
    * if one didn't appear from a SRV lookup */
   else if (lm_connection_get_server (conn->lmconn) == NULL)
@@ -1158,6 +1185,7 @@ _gabble_connection_connect (TpBaseConnection *base,
       lm_connection_set_server (conn->lmconn, priv->stream_server);
       lm_connection_set_port (conn->lmconn, priv->port);
     }
+#endif /* HAVE_LM_SRV_LOOKUPS */
 
   if (priv->https_proxy_server)
     {
@@ -1177,6 +1205,20 @@ _gabble_connection_connect (TpBaseConnection *base,
       lm_connection_set_ssl (conn->lmconn, ssl);
       lm_ssl_unref (ssl);
     }
+#ifdef HAVE_LM_STARTTLS
+  else
+    {
+      LmSSL *ssl = lm_ssl_new (NULL, connection_ssl_cb, conn, NULL);
+      lm_connection_set_ssl (conn->lmconn, ssl);
+
+      /* Try to use StartTLS if possible, but be careful about
+         allowing SSL errors in that default case. */
+      lm_ssl_use_starttls (ssl, TRUE, priv->require_encryption);
+
+      if (!priv->require_encryption)
+          priv->ignore_ssl_errors = TRUE;
+    }
+#endif /* HAVE_LM_STARTTLS */
 
   /* send whitespace to the server every 30 seconds */
   lm_connection_set_keep_alive_rate (conn->lmconn, 30);
@@ -1252,176 +1294,6 @@ connection_shut_down (TpBaseConnection *base)
     }
 }
 
-GabbleConnectionAliasSource
-_gabble_connection_get_cached_alias (GabbleConnection *conn,
-                                     TpHandle handle,
-                                     gchar **alias)
-{
-  TpBaseConnection *base = (TpBaseConnection *)conn;
-  GabbleConnectionPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE (conn,
-      GABBLE_TYPE_CONNECTION, GabbleConnectionPrivate);
-  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
-      TP_HANDLE_TYPE_CONTACT);
-  GabbleConnectionAliasSource ret = GABBLE_CONNECTION_ALIAS_NONE;
-  GabblePresence *pres;
-  const gchar *tmp;
-  gchar *user = NULL, *resource = NULL;
-
-  g_return_val_if_fail (NULL != conn, GABBLE_CONNECTION_ALIAS_NONE);
-  g_return_val_if_fail (GABBLE_IS_CONNECTION (conn), GABBLE_CONNECTION_ALIAS_NONE);
-  g_return_val_if_fail (tp_handle_is_valid (contact_handles, handle, NULL),
-      GABBLE_CONNECTION_ALIAS_NONE);
-
-  tmp = gabble_roster_handle_get_name (conn->roster, handle);
-  if (NULL != tmp)
-    {
-      ret = GABBLE_CONNECTION_ALIAS_FROM_ROSTER;
-
-      if (NULL != alias)
-        *alias = g_strdup (tmp);
-
-      goto OUT;
-    }
-
-  pres = gabble_presence_cache_get (conn->presence_cache, handle);
-  if (NULL != pres && NULL != pres->nickname)
-    {
-      ret = GABBLE_CONNECTION_ALIAS_FROM_PRESENCE;
-
-      if (NULL != alias)
-        *alias = g_strdup (pres->nickname);
-
-      goto OUT;
-    }
-
-  /* XXX: should this be more important than the ones from presence? */
-  /* if it's our own handle, use alias passed to the connmgr, if any */
-  if (handle == base->self_handle && priv->alias != NULL)
-    {
-      ret = GABBLE_CONNECTION_ALIAS_FROM_CONNMGR;
-
-      if (NULL != alias)
-        *alias = g_strdup (priv->alias);
-
-      goto OUT;
-    }
-
-  tmp = tp_handle_inspect (contact_handles, handle);
-  g_assert (NULL != tmp);
-
-  gabble_decode_jid (tmp, &user, NULL, &resource);
-
-  /* MUC handles have the nickname in the resource */
-  if (NULL != resource)
-    {
-      ret = GABBLE_CONNECTION_ALIAS_FROM_MUC_RESOURCE;
-
-      if (NULL != alias)
-        {
-          *alias = resource;
-          resource = NULL;
-        }
-
-      goto OUT;
-    }
-
-  /* if we've seen a nickname in their vCard, use that */
-  tmp = gabble_vcard_manager_get_cached_alias (conn->vcard_manager, handle);
-  if (NULL != tmp)
-    {
-      ret = GABBLE_CONNECTION_ALIAS_FROM_VCARD;
-
-      if (NULL != alias)
-        *alias = g_strdup (tmp);
-
-      goto OUT;
-    }
-
-  /* otherwise just take their local part */
-  if (NULL != user)
-    {
-      ret = GABBLE_CONNECTION_ALIAS_FROM_JID;
-
-      if (NULL != alias)
-        {
-          *alias = user;
-          user = NULL;
-        }
-
-      goto OUT;
-    }
-
-OUT:
-  g_free (user);
-  g_free (resource);
-  return ret;
-}
-
-static void
-connection_nickname_update_cb (GObject *object,
-                               TpHandle handle,
-                               gpointer user_data)
-{
-  GabbleConnection *conn = GABBLE_CONNECTION (user_data);
-  GabbleConnectionAliasSource signal_source, current_source;
-  gchar *alias = NULL;
-  GPtrArray *aliases;
-  GValue entry = { 0, };
-
-  if (object == G_OBJECT (conn->roster))
-    {
-      signal_source = GABBLE_CONNECTION_ALIAS_FROM_ROSTER;
-    }
-  else if (object == G_OBJECT (conn->presence_cache))
-    {
-      signal_source = GABBLE_CONNECTION_ALIAS_FROM_PRESENCE;
-    }
-   else if (object == G_OBJECT (conn->vcard_manager))
-     {
-       signal_source = GABBLE_CONNECTION_ALIAS_FROM_VCARD;
-     }
-  else
-    {
-      g_assert_not_reached ();
-      return;
-    }
-
-  current_source = _gabble_connection_get_cached_alias (conn, handle, &alias);
-
-  g_assert (current_source != GABBLE_CONNECTION_ALIAS_NONE);
-
-  /* if the active alias for this handle is already known and from
-   * a higher priority, this signal is not interesting so we do
-   * nothing */
-  if (signal_source < current_source)
-    {
-      DEBUG ("ignoring boring alias change for handle %u, signal from %u "
-          "but source %u has alias \"%s\"", handle, signal_source,
-          current_source, alias);
-      goto OUT;
-    }
-
-  g_value_init (&entry, GABBLE_TP_ALIAS_PAIR_TYPE);
-  g_value_take_boxed (&entry, dbus_g_type_specialized_construct
-      (GABBLE_TP_ALIAS_PAIR_TYPE));
-
-  dbus_g_type_struct_set (&entry,
-      0, handle,
-      1, alias,
-      G_MAXUINT);
-
-  aliases = g_ptr_array_sized_new (1);
-  g_ptr_array_add (aliases, g_value_get_boxed (&entry));
-
-
-  tp_svc_connection_interface_aliasing_emit_aliases_changed (conn, aliases);
-
-  g_value_unset (&entry);
-  g_ptr_array_free (aliases, TRUE);
-
-OUT:
-  g_free (alias);
-}
 
 /**
  * _gabble_connection_signal_own_presence:
@@ -1986,7 +1858,11 @@ connection_auth_cb (LmConnection *lmconn,
     }
 
 
+#ifdef HAVE_LM_CONNECTION_GET_FULL_JID
+  jid = lm_connection_get_full_jid (lmconn);
+#else
   jid = lm_connection_get_jid (lmconn);
+#endif /* HAVE_LM_CONNECTION_GET_FULL_JID */
 
   base->self_handle = tp_handle_ensure (contact_handles, jid, NULL, &error);
 
@@ -2459,6 +2335,16 @@ gabble_connection_get_capabilities (TpSvcConnectionInterfaceCapabilities *iface,
 
   tp_svc_connection_interface_capabilities_return_from_get_capabilities (
       context, ret);
+
+  for (i = 0; i < ret->len; i++)
+    {
+      GValue monster = {0, };
+
+      g_value_init (&monster, GABBLE_TP_GET_CAPABILITIES_MONSTER_TYPE);
+      g_value_take_boxed (&monster, g_ptr_array_index (ret, i));
+      g_value_unset (&monster);
+    }
+
   g_ptr_array_free (ret, TRUE);
 }
 
