@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-media-stream.h"
+
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,7 +30,6 @@
 
 #include <telepathy-glib/debug-ansi.h>
 #include "debug.h"
-#include "handles.h"
 #include "namespaces.h"
 
 #include "gabble-connection.h"
@@ -41,7 +42,6 @@
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/svc-media-interfaces.h>
 
-#include "gabble-media-stream.h"
 #include "gabble-signals-marshal.h"
 
 static void stream_handler_iface_init (gpointer, gpointer);
@@ -81,6 +81,7 @@ enum
   PROP_MEDIA_TYPE,
   PROP_CONNECTION_STATE,
   PROP_READY,
+  PROP_H263_N800_HACK,
   PROP_GOT_LOCAL_CODECS,
   PROP_SIGNALLING_STATE,
   PROP_PLAYING,
@@ -99,6 +100,7 @@ struct _GabbleMediaStreamPrivate
   gchar *object_path;
   guint id;
   guint media_type;
+  gboolean h283_n800_hack;
 
   gboolean ready;
   gboolean sending;
@@ -147,21 +149,21 @@ gabble_media_stream_init (GabbleMediaStream *self)
 
   self->priv = priv;
 
-  g_value_init (&priv->native_codecs, TP_TYPE_CODEC_LIST);
+  g_value_init (&priv->native_codecs, GABBLE_TP_TYPE_CODEC_LIST);
   g_value_take_boxed (&priv->native_codecs,
-      dbus_g_type_specialized_construct (TP_TYPE_CODEC_LIST));
+      dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CODEC_LIST));
 
-  g_value_init (&priv->native_candidates, TP_TYPE_CANDIDATE_LIST);
+  g_value_init (&priv->native_candidates, GABBLE_TP_TYPE_CANDIDATE_LIST);
   g_value_take_boxed (&priv->native_candidates,
-      dbus_g_type_specialized_construct (TP_TYPE_CANDIDATE_LIST));
+      dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CANDIDATE_LIST));
 
-  g_value_init (&priv->remote_codecs, TP_TYPE_CODEC_LIST);
+  g_value_init (&priv->remote_codecs, GABBLE_TP_TYPE_CODEC_LIST);
   g_value_take_boxed (&priv->remote_codecs,
-      dbus_g_type_specialized_construct (TP_TYPE_CODEC_LIST));
+      dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CODEC_LIST));
 
-  g_value_init (&priv->remote_candidates, TP_TYPE_CANDIDATE_LIST);
+  g_value_init (&priv->remote_candidates, GABBLE_TP_TYPE_CANDIDATE_LIST);
   g_value_take_boxed (&priv->remote_candidates,
-      dbus_g_type_specialized_construct (TP_TYPE_CANDIDATE_LIST));
+      dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CANDIDATE_LIST));
 }
 
 static GObject *
@@ -218,6 +220,9 @@ gabble_media_stream_get_property (GObject    *object,
     case PROP_MEDIA_TYPE:
       g_value_set_uint (value, priv->media_type);
       break;
+    case PROP_H263_N800_HACK:
+      g_value_set_boolean (value, priv->h283_n800_hack);
+      break;
     case PROP_CONNECTION_STATE:
       g_value_set_uint (value, stream->connection_state);
       break;
@@ -278,6 +283,9 @@ gabble_media_stream_set_property (GObject      *object,
     case PROP_MEDIA_TYPE:
       priv->media_type = g_value_get_uint (value);
       break;
+    case PROP_H263_N800_HACK:
+      priv->h283_n800_hack = g_value_get_boolean (value);
+      break;
     case PROP_CONNECTION_STATE:
       GMS_DEBUG_INFO (priv->session, "stream %s connection state %d",
           stream->name, stream->connection_state);
@@ -325,7 +333,8 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_stream_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_media_stream_class, sizeof (GabbleMediaStreamPrivate));
+  g_type_class_add_private (gabble_media_stream_class,
+      sizeof (GabbleMediaStreamPrivate));
 
   object_class->constructor = gabble_media_stream_constructor;
 
@@ -345,15 +354,14 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
                                     G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
-  param_spec = g_param_spec_object ("media-session", "GabbleMediaSession object",
-                                    "Gabble media session object that owns this "
-                                    "media stream object.",
-                                    GABBLE_TYPE_MEDIA_SESSION,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_MEDIA_SESSION, param_spec);
+  param_spec = g_param_spec_object ("media-session",
+      "GabbleMediaSession object",
+      "Gabble media session object that owns this media stream object.",
+      GABBLE_TYPE_MEDIA_SESSION,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_NICK |
+      G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_MEDIA_SESSION,
+      param_spec);
 
   param_spec = g_param_spec_string ("object-path", "D-Bus object path",
                                     "The D-Bus object path used for this "
@@ -368,7 +376,7 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
   param_spec = g_param_spec_enum ("mode", "Signalling mode",
                                   "Which signalling mode used to control the "
                                   "stream.",
-                                  gabble_media_session_mode_get_type(),
+                                  gabble_media_session_mode_get_type (),
                                   MODE_JINGLE,
                                   G_PARAM_CONSTRUCT_ONLY |
                                   G_PARAM_READWRITE |
@@ -377,13 +385,9 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
   g_object_class_install_property (object_class, PROP_MODE, param_spec);
 
   param_spec = g_param_spec_string ("name", "Stream name",
-                                    "An opaque name for the stream used in the "
-                                    "signalling.",
-                                    NULL,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NAME |
-                                    G_PARAM_STATIC_BLURB);
+      "An opaque name for the stream used in the signalling.", NULL,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_NAME |
+      G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_NAME, param_spec);
 
   param_spec = g_param_spec_uint ("id", "Stream ID",
@@ -420,6 +424,19 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
                                   G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_MEDIA_TYPE, param_spec);
 
+  param_spec = g_param_spec_boolean ("h263-n800-hack", "Enable H263-N800 hack",
+                                     "A boolean signifying whether this "
+                                     "stream should dynamically rewrite "
+                                     "codec names between H263-N800 and "
+                                     "H263-1998.",
+                                     FALSE,
+                                     G_PARAM_CONSTRUCT_ONLY |
+                                     G_PARAM_READWRITE |
+                                     G_PARAM_STATIC_NAME |
+                                     G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_H263_N800_HACK,
+      param_spec);
+
   param_spec = g_param_spec_uint ("connection-state", "Stream connection state",
                                   "An integer indicating the state of the"
                                   "stream's connection.",
@@ -430,7 +447,8 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
                                   G_PARAM_READWRITE |
                                   G_PARAM_STATIC_NAME |
                                   G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_CONNECTION_STATE, param_spec);
+  g_object_class_install_property (object_class, PROP_CONNECTION_STATE,
+      param_spec);
 
   param_spec = g_param_spec_boolean ("ready", "Ready?",
                                      "A boolean signifying whether the user "
@@ -444,14 +462,12 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
   g_object_class_install_property (object_class, PROP_READY, param_spec);
 
   param_spec = g_param_spec_boolean ("got-local-codecs", "Got local codecs?",
-                                     "A boolean signifying whether we've got "
-                                     "the locally supported codecs from the user.",
-                                     FALSE,
-                                     G_PARAM_CONSTRUCT |
-                                     G_PARAM_READWRITE |
-                                     G_PARAM_STATIC_NAME |
-                                     G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_GOT_LOCAL_CODECS, param_spec);
+      "A boolean signifying whether we've got the locally supported codecs "
+      "from the user.", FALSE,
+      G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_NAME |
+      G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_GOT_LOCAL_CODECS,
+      param_spec);
 
   param_spec = g_param_spec_uint ("signalling-state", "Signalling state",
                                   "Whether the stream is newly created, "
@@ -463,7 +479,8 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
                                   G_PARAM_READWRITE |
                                   G_PARAM_STATIC_NAME |
                                   G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_SIGNALLING_STATE, param_spec);
+  g_object_class_install_property (object_class, PROP_SIGNALLING_STATE,
+      param_spec);
 
   param_spec = g_param_spec_boolean ("playing", "Set playing",
                                      "A boolean signifying whether the stream "
@@ -515,7 +532,7 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
                   0,
                   NULL, NULL,
                   gabble_marshal_VOID__STRING_BOXED,
-                  G_TYPE_NONE, 2, G_TYPE_STRING, TP_TYPE_TRANSPORT_LIST);
+                  G_TYPE_NONE, 2, G_TYPE_STRING, GABBLE_TP_TYPE_TRANSPORT_LIST);
 
   signals[SUPPORTED_CODECS] =
     g_signal_new ("supported-codecs",
@@ -524,7 +541,7 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
                   0,
                   NULL, NULL,
                   g_cclosure_marshal_VOID__BOXED,
-                  G_TYPE_NONE, 1, TP_TYPE_CODEC_LIST);
+                  G_TYPE_NONE, 1, GABBLE_TP_TYPE_CODEC_LIST);
 
   signals[ERROR] =
     g_signal_new ("error",
@@ -606,7 +623,9 @@ gabble_media_stream_error (GabbleMediaStream *self,
 
   priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (self);
 
-  GMS_DEBUG_WARNING (priv->session, "Media.StreamHandler::Error called, error %u (%s) -- emitting signal", errno, message);
+  GMS_DEBUG_WARNING (priv->session,
+      "Media.StreamHandler::Error called, error %u (%s) -- emitting signal",
+      errno, message);
 
   g_signal_emit (self, signals[ERROR], 0, errno, message);
 
@@ -627,7 +646,7 @@ gabble_media_stream_error_async (TpSvcMediaStreamHandler *iface,
                                  DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
-  GError *error;
+  GError *error = NULL;
 
   if (gabble_media_stream_error (self, errno, message, &error))
     {
@@ -725,9 +744,9 @@ gabble_media_stream_new_native_candidate (TpSvcMediaStreamHandler *iface,
 
   candidates = g_value_get_boxed (&priv->native_candidates);
 
-  g_value_init (&candidate, TP_TYPE_CANDIDATE_STRUCT);
+  g_value_init (&candidate, GABBLE_TP_TYPE_CANDIDATE_STRUCT);
   g_value_take_boxed (&candidate,
-      dbus_g_type_specialized_construct (TP_TYPE_CANDIDATE_STRUCT));
+      dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CANDIDATE_STRUCT));
 
   dbus_g_type_struct_set (&candidate,
       0, candidate_id,
@@ -749,15 +768,16 @@ gabble_media_stream_new_native_candidate (TpSvcMediaStreamHandler *iface,
   addr = g_value_get_string (g_value_array_get_nth (transport, 1));
   if (!strcmp (addr, "127.0.0.1"))
     {
-      GMS_DEBUG_WARNING (priv->session, "%s: ignoring native localhost candidate",
-                         G_STRFUNC);
+      GMS_DEBUG_WARNING (priv->session,
+          "%s: ignoring native localhost candidate", G_STRFUNC);
       tp_svc_media_stream_handler_return_from_new_native_candidate (context);
       return;
     }
 
   g_ptr_array_add (candidates, g_value_get_boxed (&candidate));
 
-  GMS_DEBUG_INFO (priv->session, "put 1 native candidate from stream-engine into cache");
+  GMS_DEBUG_INFO (priv->session,
+      "put 1 native candidate from stream-engine into cache");
 
   push_native_candidates (self);
 
@@ -894,7 +914,8 @@ candidates_msg_reply_cb (GabbleConnection *conn,
 
   priv = GABBLE_MEDIA_STREAM_GET_PRIVATE (stream);
 
-  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (priv->session, "candidates failed");
+  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (priv->session,
+      "candidates failed");
 
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
@@ -909,8 +930,7 @@ _add_rtp_candidate_node (GabbleMediaSession *session, LmMessageNode *parent,
   gchar *port_str;
   gchar *pref_str;
   gchar *xml;
-  const gchar *type_str;
-  const gchar *candidate_id;
+  const gchar *type_str, *proto_str, *candidate_id;
   guint port;
   gdouble pref;
   TpMediaStreamBaseProto proto;
@@ -925,7 +945,7 @@ _add_rtp_candidate_node (GabbleMediaSession *session, LmMessageNode *parent,
   /* jingle audio only supports the concept of one transport per candidate */
   g_assert (transports->len == 1);
 
-  g_value_init (&transport, TP_TYPE_TRANSPORT_STRUCT);
+  g_value_init (&transport, GABBLE_TP_TYPE_TRANSPORT_STRUCT);
   g_value_set_static_boxed (&transport, g_ptr_array_index (transports, 0));
 
   dbus_g_type_struct_get (&transport,
@@ -952,11 +972,28 @@ _add_rtp_candidate_node (GabbleMediaSession *session, LmMessageNode *parent,
       type_str = "relay";
       break;
     default:
-      g_error ("%s: TpMediaStreamTransportType has an invalid value",
-        G_STRFUNC);
-      return;
+      g_warning ("%s: TpMediaStreamTransportType has an invalid value, "
+          "ignoring candidate", G_STRFUNC);
+      goto out;
   }
 
+  switch (proto)
+    {
+    case TP_MEDIA_STREAM_BASE_PROTO_UDP:
+      proto_str = "udp";
+      break;
+    case TP_MEDIA_STREAM_BASE_PROTO_TCP:
+      if (port == 443 && type == TP_MEDIA_STREAM_TRANSPORT_TYPE_RELAY)
+        proto_str = "ssltcp";
+      else
+        proto_str = "tcp";
+      break;
+    default:
+      g_warning ("%s: TpMediaStreamBaseProto has an invalid value, ignoring "
+          "candidate", G_STRFUNC);
+      goto out;
+    }
+
   cand_node = lm_message_node_add_child (parent, "candidate", NULL);
   lm_message_node_set_attributes (cand_node,
       "name", "rtp",
@@ -965,7 +1002,7 @@ _add_rtp_candidate_node (GabbleMediaSession *session, LmMessageNode *parent,
       "username", user,
       "password", pass,
       "preference", pref_str,
-      "protocol", (proto == TP_MEDIA_STREAM_BASE_PROTO_UDP) ? "udp" : "tcp",
+      "protocol", proto_str,
       "type", type_str,
       "network", "0",
       "generation", "0",
@@ -975,13 +1012,14 @@ _add_rtp_candidate_node (GabbleMediaSession *session, LmMessageNode *parent,
   GMS_DEBUG_DUMP (session,
     "  from Telepathy D-Bus struct: [%s\"%s\", %s[%s1, \"%s\", %d, %s, "
     "\"%s\", \"%s\", %f, %s, \"%s\", \"%s\"%s]]",
-    TP_ANSI_BOLD_OFF, candidate_id, TP_ANSI_BOLD_ON, TP_ANSI_BOLD_OFF, addr, port,
-    tp_protocols[proto], "RTP", "AVP", pref, tp_transports[type], user, pass,
-    TP_ANSI_BOLD_ON);
+    TP_ANSI_BOLD_OFF, candidate_id, TP_ANSI_BOLD_ON, TP_ANSI_BOLD_OFF,
+    addr, port, tp_protocols[proto], "RTP", "AVP", pref, tp_transports[type],
+    user, pass, TP_ANSI_BOLD_ON);
   GMS_DEBUG_DUMP (session,
     "  to Jingle XML: [%s%s%s]", TP_ANSI_BOLD_OFF, xml, TP_ANSI_BOLD_ON);
   g_free (xml);
 
+out:
   g_free (addr);
   g_free (user);
   g_free (pass);
@@ -1064,7 +1102,7 @@ push_native_candidates (GabbleMediaStream *stream)
     push_candidate (stream, g_ptr_array_index (candidates, i));
 
   g_value_take_boxed (&priv->native_candidates,
-    dbus_g_type_specialized_construct (TP_TYPE_CANDIDATE_LIST));
+    dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CANDIDATE_LIST));
 }
 
 void
@@ -1121,7 +1159,7 @@ _gabble_media_stream_post_remote_codecs (GabbleMediaStream *stream,
           return FALSE;
         }
 
-      id = atoi(str);
+      id = atoi (str);
 
       /* codec name */
       name = lm_message_node_get_attribute (node, "name");
@@ -1130,6 +1168,24 @@ _gabble_media_stream_post_remote_codecs (GabbleMediaStream *stream,
           name = "";
         }
 
+      /* if the N800 hack mode is enabled then skip any codecs called
+       * H263-N800, and interpret the remote end's H263-1998 as H263-N800 */
+      if (priv->h283_n800_hack)
+        {
+          if (!tp_strdiff (name, "H263-N800"))
+            {
+              GMS_DEBUG_WARNING (priv->session, "N800 hack enabled, skipping "
+                  "unexpected remote H263-N800!");
+              continue;
+            }
+          else if (!tp_strdiff (name, "H263-1998"))
+            {
+              GMS_DEBUG_INFO (priv->session, "N800 hack enabled, interpeting "
+                  "remote H263-1998 as H263-N800");
+              name = "H263-N800";
+            }
+        }
+
       /* clock rate: jingle and newer GTalk */
       str = lm_message_node_get_attribute (node, "clockrate"); /* google */
       if (str == NULL)
@@ -1164,14 +1220,14 @@ _gabble_media_stream_post_remote_codecs (GabbleMediaStream *stream,
           g_hash_table_insert (params, "bitrate", g_strdup (str));
         }
 
-      g_value_init (&codec, TP_TYPE_CODEC_STRUCT);
+      g_value_init (&codec, GABBLE_TP_TYPE_CODEC_STRUCT);
       g_value_take_boxed (&codec,
-          dbus_g_type_specialized_construct (TP_TYPE_CODEC_STRUCT));
+          dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CODEC_STRUCT));
 
       dbus_g_type_struct_set (&codec,
           0, id,
           1, name,
-          2, TP_MEDIA_STREAM_TYPE_AUDIO,
+          2, priv->media_type,
           3, clockrate,
           4, channels,
           5, params,
@@ -1211,7 +1267,7 @@ push_remote_codecs (GabbleMediaStream *stream)
   tp_svc_media_stream_handler_emit_set_remote_codecs (stream, codecs);
 
   g_value_take_boxed (&priv->remote_codecs,
-      dbus_g_type_specialized_construct (TP_TYPE_CODEC_LIST));
+      dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CODEC_LIST));
 }
 
 gboolean
@@ -1285,13 +1341,25 @@ _gabble_media_stream_post_remote_candidates (GabbleMediaStream *stream,
         }
       else if (strcmp (str, "tcp") == 0)
         {
+          if (port == 443)
+            {
+              GMS_DEBUG_WARNING (priv->session, "%s: tcp candidates on port "
+                  "443 must be ssltcp, ignoring candidate", G_STRFUNC);
+              continue;
+            }
+
           proto = TP_MEDIA_STREAM_BASE_PROTO_TCP;
         }
       else if (strcmp (str, "ssltcp") == 0)
         {
-          GMS_DEBUG_WARNING (priv->session, "%s: ssltcp candidates "
-                             "not yet supported", G_STRFUNC);
-          continue;
+          if (port != 443)
+            {
+              GMS_DEBUG_WARNING (priv->session, "%s: ssltcp candidates must "
+                  "be on port 443, ignoring candidate", G_STRFUNC);
+              continue;
+            }
+
+          proto = TP_MEDIA_STREAM_BASE_PROTO_TCP;
         }
       else
         goto FAILURE;
@@ -1347,9 +1415,9 @@ _gabble_media_stream_post_remote_candidates (GabbleMediaStream *stream,
       gen = atoi (str);
 
 
-      g_value_init (&transport, TP_TYPE_TRANSPORT_STRUCT);
+      g_value_init (&transport, GABBLE_TP_TYPE_TRANSPORT_STRUCT);
       g_value_take_boxed (&transport,
-          dbus_g_type_specialized_construct (TP_TYPE_TRANSPORT_STRUCT));
+          dbus_g_type_specialized_construct (GABBLE_TP_TYPE_TRANSPORT_STRUCT));
 
       dbus_g_type_struct_set (&transport,
           0, 1,         /* component number */
@@ -1368,9 +1436,9 @@ _gabble_media_stream_post_remote_candidates (GabbleMediaStream *stream,
       g_ptr_array_add (transports, g_value_get_boxed (&transport));
 
 
-      g_value_init (&candidate, TP_TYPE_CANDIDATE_STRUCT);
+      g_value_init (&candidate, GABBLE_TP_TYPE_CANDIDATE_STRUCT);
       g_value_take_boxed (&candidate,
-          dbus_g_type_specialized_construct (TP_TYPE_CANDIDATE_STRUCT));
+          dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CANDIDATE_STRUCT));
 
       /* FIXME: is this naming scheme sensible? */
       candidate_id = g_strdup_printf ("R%d", ++priv->remote_candidate_count);
@@ -1383,12 +1451,17 @@ _gabble_media_stream_post_remote_candidates (GabbleMediaStream *stream,
       g_ptr_array_add (candidates, g_value_get_boxed (&candidate));
 
       xml = lm_message_node_to_string (node);
-      GMS_DEBUG_INFO (priv->session, "put 1 remote candidate from peer into cache");
+      GMS_DEBUG_INFO (priv->session,
+          "put 1 remote candidate from peer into cache");
       GMS_DEBUG_DUMP (priv->session, "  from Jingle XML: [%s%s%s]",
                       TP_ANSI_BOLD_OFF, xml, TP_ANSI_BOLD_ON);
-      GMS_DEBUG_DUMP (priv->session, "  to Telepathy D-Bus struct: [%s\"%s\", %s[%s1, \"%s\", %d, %s, \"%s\", \"%s\", %f, %s, \"%s\", \"%s\"%s]]",
-                      TP_ANSI_BOLD_OFF, candidate_id, TP_ANSI_BOLD_ON,
-                      TP_ANSI_BOLD_OFF, addr, port, tp_protocols[proto], "RTP", "AVP", pref, tp_transports[type], user, pass, TP_ANSI_BOLD_ON);
+      GMS_DEBUG_DUMP (priv->session,
+          "  to Telepathy D-Bus struct: [%s\"%s\", %s[%s1, \"%s\", %d, %s, "
+          "\"%s\", \"%s\", %f, %s, \"%s\", \"%s\"%s]]",
+          TP_ANSI_BOLD_OFF, candidate_id, TP_ANSI_BOLD_ON,
+          TP_ANSI_BOLD_OFF, addr, port, tp_protocols[proto],
+          "RTP", "AVP", pref, tp_transports[type],
+          user, pass, TP_ANSI_BOLD_ON);
       g_free (xml);
 
       g_free (candidate_id);
@@ -1442,7 +1515,7 @@ push_remote_candidates (GabbleMediaStream *stream)
     }
 
   g_value_take_boxed (&priv->remote_candidates,
-      dbus_g_type_specialized_construct (TP_TYPE_CANDIDATE_LIST));
+      dbus_g_type_specialized_construct (GABBLE_TP_TYPE_CANDIDATE_LIST));
 }
 
 static void
@@ -1599,7 +1672,7 @@ _gabble_media_stream_content_node_add_description (GabbleMediaStream *stream,
       LmMessageNode *pt_node;
       CodecParamsFromTpContext ctx;
 
-      g_value_init (&codec, TP_TYPE_CODEC_STRUCT);
+      g_value_init (&codec, GABBLE_TP_TYPE_CODEC_STRUCT);
       g_value_set_static_boxed (&codec, g_ptr_array_index (codecs, i));
 
       dbus_g_type_struct_get (&codec,
@@ -1610,6 +1683,26 @@ _gabble_media_stream_content_node_add_description (GabbleMediaStream *stream,
           5, &params,
           G_MAXUINT);
 
+      /* if the N800 hack mode is enabled then skip any codecs called
+       * H263-1998, and rebrand H263-N800 as H263-1998 */
+      if (priv->h283_n800_hack)
+        {
+          if (!tp_strdiff (name, "H263-1998"))
+            {
+              GMS_DEBUG_INFO (priv->session, "N800 hack enabled, skipping "
+                  "real local H263-1998");
+              g_free (name);
+              continue;
+            }
+          else if (!tp_strdiff (name, "H263-N800"))
+            {
+              GMS_DEBUG_INFO (priv->session, "N800 hack enabled, advertising "
+                  "local H263-N800 as H263-1998");
+              g_free (name);
+              name = g_strdup ("H263-1998");
+            }
+        }
+
       /* create a sub-node called "payload-type" and fill it */
       pt_node = lm_message_node_add_child (desc_node, "payload-type", NULL);
 
@@ -1698,7 +1791,8 @@ _gabble_media_stream_update_sending (GabbleMediaStream *stream,
 static void
 stream_handler_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcMediaStreamHandlerClass *klass = (TpSvcMediaStreamHandlerClass *)g_iface;
+  TpSvcMediaStreamHandlerClass *klass =
+    (TpSvcMediaStreamHandlerClass *)g_iface;
 
 #define IMPLEMENT(x,suffix) tp_svc_media_stream_handler_implement_##x (\
     klass, gabble_media_stream_##x##suffix)
