@@ -27,9 +27,14 @@
 #include <string.h>
 #include <time.h>
 
+#include <telepathy-glib/dbus.h>
+#include <telepathy-glib/debug-ansi.h>
+#include <telepathy-glib/errors.h>
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/channel-iface.h>
+
 #define DEBUG_FLAG GABBLE_DEBUG_MUC
 
-#include <telepathy-glib/debug-ansi.h>
 #include "debug.h"
 #include "disco.h"
 #include "gabble-connection.h"
@@ -37,11 +42,6 @@
 #include "namespaces.h"
 #include "util.h"
 
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/errors.h>
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/channel-iface.h>
-
 #define DEFAULT_JOIN_TIMEOUT (180 * 1000)
 #define MAX_NICK_RETRIES 3
 
@@ -75,6 +75,9 @@ enum
 {
     READY,
     JOIN_ERROR,
+    PRE_INVITE,
+    CONTACT_JOIN,
+    PRE_PRESENCE,
     LAST_SIGNAL
 };
 
@@ -93,14 +96,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef enum {
-    MUC_STATE_CREATED = 0,
-    MUC_STATE_INITIATED,
-    MUC_STATE_AUTH,
-    MUC_STATE_JOINED,
-    MUC_STATE_ENDED,
-} GabbleMucState;
-
 #ifdef ENABLE_DEBUG
 static const gchar *muc_states[] =
 {
@@ -156,6 +151,7 @@ enum
 {
   ROOM_PROP_ANONYMOUS = 0,
   ROOM_PROP_INVITE_ONLY,
+  ROOM_PROP_INVITE_RESTRICTED,
   ROOM_PROP_MODERATED,
   ROOM_PROP_NAME,
   ROOM_PROP_DESCRIPTION,
@@ -175,6 +171,7 @@ enum
 const TpPropertySignature room_property_signatures[NUM_ROOM_PROPS] = {
       { "anonymous",         G_TYPE_BOOLEAN },  /* impl: READ, WRITE */
       { "invite-only",       G_TYPE_BOOLEAN },  /* impl: READ, WRITE */
+      { "invite-restricted", G_TYPE_BOOLEAN },  /* impl: WRITE */
       { "moderated",         G_TYPE_BOOLEAN },  /* impl: READ, WRITE */
       { "name",              G_TYPE_STRING },   /* impl: READ, WRITE */
       { "description",       G_TYPE_STRING },   /* impl: READ, WRITE */
@@ -599,6 +596,27 @@ contact_handle_to_room_identity (GabbleMucChannel *chan, TpHandle main_handle,
   g_free (jid);
 }
 
+static LmMessage *
+create_presence_message (GabbleMucChannel *self,
+                         LmMessageSubType sub_type,
+                         LmMessageNode **x_node)
+{
+  GabbleMucChannelPrivate *priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (self);
+  LmMessage *msg;
+  LmMessageNode *node;
+
+  msg = lm_message_new_with_sub_type (priv->self_jid->str,
+      LM_MESSAGE_TYPE_PRESENCE, sub_type);
+
+  node = lm_message_node_add_child (msg->node, "x", NULL);
+  lm_message_node_set_attribute (node, "xmlns", NS_MUC);
+
+  if (x_node != NULL)
+    *x_node = node;
+
+  return msg;
+}
+
 static gboolean
 send_join_request (GabbleMucChannel *channel,
                    const gchar *password,
@@ -612,10 +630,8 @@ send_join_request (GabbleMucChannel *channel,
   priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (channel);
 
   /* build the message */
-  msg = lm_message_new (priv->self_jid->str, LM_MESSAGE_TYPE_PRESENCE);
-
-  x_node = lm_message_node_add_child (msg->node, "x", NULL);
-  lm_message_node_set_attribute (x_node, "xmlns", NS_MUC);
+  msg = create_presence_message (channel, LM_MESSAGE_SUB_TYPE_NOT_SET,
+      &x_node);
 
   g_free (priv->password);
 
@@ -625,6 +641,8 @@ send_join_request (GabbleMucChannel *channel,
       lm_message_node_add_child (x_node, "password", password);
     }
 
+  g_signal_emit (channel, signals[PRE_PRESENCE], 0, msg);
+
   /* send it */
   ret = _gabble_connection_send (priv->conn, msg, error);
   if (!ret)
@@ -653,15 +671,16 @@ send_leave_message (GabbleMucChannel *channel,
   priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (channel);
 
   /* build the message */
-  msg = lm_message_new_with_sub_type (priv->self_jid->str,
-                                      LM_MESSAGE_TYPE_PRESENCE,
-                                      LM_MESSAGE_SUB_TYPE_UNAVAILABLE);
+  msg = create_presence_message (channel, LM_MESSAGE_SUB_TYPE_UNAVAILABLE,
+      NULL);
 
   if (reason != NULL)
     {
       lm_message_node_add_child (msg->node, "status", reason);
     }
 
+  g_signal_emit (channel, signals[PRE_PRESENCE], 0, msg);
+
   /* send it */
   ret = _gabble_connection_send (priv->conn, msg, &error);
   if (!ret)
@@ -835,6 +854,33 @@ gabble_muc_channel_class_init (GabbleMucChannelClass *gabble_muc_channel_class)
                   g_cclosure_marshal_VOID__POINTER,
                   G_TYPE_NONE, 1, G_TYPE_POINTER);
 
+  signals[PRE_INVITE] =
+    g_signal_new ("pre-invite",
+                  G_OBJECT_CLASS_TYPE (gabble_muc_channel_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__UINT,
+                  G_TYPE_NONE, 1, G_TYPE_UINT);
+
+  signals[CONTACT_JOIN] =
+    g_signal_new ("contact-join",
+                  G_OBJECT_CLASS_TYPE (gabble_muc_channel_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__UINT,
+                  G_TYPE_NONE, 1, G_TYPE_UINT);
+
+  signals[PRE_PRESENCE] =
+    g_signal_new ("pre-presence",
+                  G_OBJECT_CLASS_TYPE (gabble_muc_channel_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__UINT,
+                  G_TYPE_NONE, 1, G_TYPE_UINT);
+
   tp_group_mixin_class_init (object_class,
                                  G_STRUCT_OFFSET (GabbleMucChannelClass,
                                    group_class),
@@ -1045,12 +1091,15 @@ channel_state_changed (GabbleMucChannel *chan,
 
   if (new_state == MUC_STATE_JOINED || new_state == MUC_STATE_AUTH)
     {
+      TpBaseConnection *base = (TpBaseConnection *) priv->conn;
+
       if (!priv->ready_emitted)
         {
           g_signal_emit (chan, signals[READY], 0);
 
           priv->ready_emitted = TRUE;
         }
+      g_signal_emit (chan, signals[CONTACT_JOIN], 0, base->self_handle);
     }
 }
 
@@ -1204,16 +1253,8 @@ _gabble_muc_channel_presence_error (GabbleMucChannel *chan,
             }
           break;
         default:
-          if (error != INVALID_XMPP_ERROR)
-            {
-              tp_error = g_error_new (TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-                                      gabble_xmpp_error_description (error));
-            }
-          else
-            {
-              tp_error = g_error_new (TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-                                      "unknown error");
-            }
+          tp_error = g_error_new (TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+              gabble_xmpp_error_description (error));
           break;
       }
 
@@ -1457,6 +1498,10 @@ update_permissions (GabbleMucChannel *chan)
       ROOM_PROP_INVITE_ONLY, prop_flags_add, prop_flags_rem,
       changed_props_flags);
 
+  tp_properties_mixin_change_flags (G_OBJECT (chan),
+      ROOM_PROP_INVITE_RESTRICTED, prop_flags_add, prop_flags_rem,
+      changed_props_flags);
+
   tp_properties_mixin_change_flags (G_OBJECT (chan),
       ROOM_PROP_MODERATED, prop_flags_add, prop_flags_rem,
       changed_props_flags);
@@ -1613,6 +1658,8 @@ _gabble_muc_channel_member_presence_updated (GabbleMucChannel *chan,
                    */
                   tp_group_mixin_change_flags ((GObject *) chan, 0,
                       TP_CHANNEL_GROUP_FLAG_HANDLE_OWNERS_NOT_AVAILABLE);
+
+                  g_signal_emit (chan, signals[CONTACT_JOIN], 0, owner_handle);
                 }
             }
 
@@ -2266,6 +2313,8 @@ gabble_muc_channel_add_member (GObject *obj,
       return FALSE;
     }
 
+  g_signal_emit (obj, signals[PRE_INVITE], 0, handle);
+
   msg = lm_message_new (priv->jid, LM_MESSAGE_TYPE_MESSAGE);
 
   x_node = lm_message_node_add_child (msg->node, "x", NULL);
@@ -2561,6 +2610,11 @@ request_config_form_reply_cb (GabbleConnection *conn, LmMessage *sent_msg,
         {
           id = ROOM_PROP_INVITE_ONLY;
         }
+      else if (strcmp (var, "muc#roomconfig_allowinvites") == 0)
+        {
+          id = ROOM_PROP_INVITE_RESTRICTED;
+          invert = TRUE;
+        }
       else if (strcmp (var, "moderated") == 0 ||
                strcmp (var, "muc#roomconfig_moderatedroom") == 0 ||
                strcmp (var, "muc#owner_moderatedroom") == 0)
@@ -2612,7 +2666,8 @@ request_config_form_reply_cb (GabbleConnection *conn, LmMessage *sent_msg,
           continue;
         }
 
-      DEBUG ("looking up %s", room_property_signatures[id].name);
+      DEBUG ("looking up %s... has=%d", room_property_signatures[id].name,
+          tp_properties_context_has (ctx, id));
 
       if (!tp_properties_context_has (ctx, id))
         continue;
@@ -2637,6 +2692,7 @@ request_config_form_reply_cb (GabbleConnection *conn, LmMessage *sent_msg,
           }
         }
 
+      DEBUG ("Setting value %s for %s", val_str, var);
       lm_message_node_set_value (value_node, val_str);
 
       props_left &= ~(1 << id);
@@ -2784,6 +2840,22 @@ gabble_muc_channel_set_chat_state (TpSvcChannelInterfaceChatState *iface,
   tp_svc_channel_interface_chat_state_return_from_set_chat_state (context);
 }
 
+gboolean
+gabble_muc_channel_send_presence (GabbleMucChannel *self,
+                                  GError **error)
+{
+  GabbleMucChannelPrivate *priv = GABBLE_MUC_CHANNEL_GET_PRIVATE (self);
+  LmMessage *msg;
+  gboolean result;
+
+  msg = create_presence_message (self, LM_MESSAGE_SUB_TYPE_NOT_SET, NULL);
+  g_signal_emit (self, signals[PRE_PRESENCE], 0, msg);
+  result = _gabble_connection_send (priv->conn, msg, error);
+
+  lm_message_unref (msg);
+  return result;
+}
+
 static void
 channel_iface_init (gpointer g_iface, gpointer iface_data)
 {
