@@ -20,6 +20,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-register.h"
+
 #define DBUS_API_SUBJECT_TO_CHANGE
 
 #include <dbus/dbus-glib.h>
@@ -34,7 +36,6 @@
 
 #include "gabble-connection.h"
 #include "gabble-error.h"
-#include "gabble-register.h"
 #include "gabble-signals-marshal.h"
 #include "namespaces.h"
 #include "util.h"
@@ -67,7 +68,9 @@ struct _GabbleRegisterPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_REGISTER_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_REGISTER, GabbleRegisterPrivate))
+#define GABBLE_REGISTER_GET_PRIVATE(o) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_REGISTER,\
+                                GabbleRegisterPrivate))
 
 static void
 gabble_register_init (GabbleRegister *obj)
@@ -87,7 +90,8 @@ gabble_register_class_init (GabbleRegisterClass *gabble_register_class)
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_register_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_register_class, sizeof (GabbleRegisterPrivate));
+  g_type_class_add_private (gabble_register_class,
+      sizeof (GabbleRegisterPrivate));
 
   object_class->get_property = gabble_register_get_property;
   object_class->set_property = gabble_register_set_property;
@@ -219,11 +223,8 @@ nokia_iv_set_reply_cb (GabbleConnection *conn,
 
           error = gabble_xmpp_error_from_node (node);
 
-          if (error != INVALID_XMPP_ERROR)
-            {
-              g_string_append_printf (msg, ": %s",
-                  gabble_xmpp_error_string (error));
-            }
+          g_string_append_printf (msg, ": %s",
+              gabble_xmpp_error_string (error));
         }
 
       g_signal_emit (object, signals[FINISHED], 0, FALSE, code, msg->str);
@@ -302,16 +303,16 @@ nokia_iv_get_reply_cb (GabbleConnection *conn,
   for (i = 0; i < strlen (challenge); i++)
     challenge[i] = tolower (challenge[i]);
 
-  md5_init(&calculator);
-  md5_append(&calculator, (const md5_byte_t *)auth_btid,
-             strlen(auth_btid));
-  md5_append(&calculator, (const md5_byte_t *)":", 1);
-  md5_append(&calculator, (const md5_byte_t *)challenge, strlen(challenge));
-  md5_finish(&calculator, digest);
+  md5_init (&calculator);
+  md5_append (&calculator, (const md5_byte_t *)auth_btid,
+             strlen (auth_btid));
+  md5_append (&calculator, (const md5_byte_t *)":", 1);
+  md5_append (&calculator, (const md5_byte_t *)challenge, strlen (challenge));
+  md5_finish (&calculator, digest);
 
   for (i = 0; i < 16; i++)
     {
-      sprintf(response + i*2, "%02x",digest[i]);
+      sprintf (response + i*2, "%02x",digest[i]);
     }
 
   lm_message_node_add_child (query_node, "mac", auth_mac);
@@ -377,11 +378,8 @@ set_reply_cb (GabbleConnection *conn,
               code = TP_ERROR_INVALID_ARGUMENT;
             }
 
-          if (error != INVALID_XMPP_ERROR)
-            {
-              g_string_append_printf (msg, ": %s",
-                  gabble_xmpp_error_string (error));
-            }
+          g_string_append_printf (msg, ": %s",
+              gabble_xmpp_error_string (error));
         }
 
       g_signal_emit (object, signals[FINISHED], 0, FALSE, code, msg->str);
@@ -484,7 +482,7 @@ send_registration (GabbleRegister *reg, RegistrationStage stage)
   GabbleRegisterPrivate *priv = GABBLE_REGISTER_GET_PRIVATE (reg);
   LmMessage *msg;
   LmMessageNode *node;
-  GError *error;
+  GError *error = NULL;
   GabbleConnectionMsgReplyFunc handler;
 
   msg = lm_message_new_with_sub_type (NULL, LM_MESSAGE_TYPE_IQ,
