@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-roomlist-channel.h"
+
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,7 +30,6 @@
 #include "debug.h"
 #include "disco.h"
 #include "gabble-connection.h"
-#include "handles.h"
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/interfaces.h>
@@ -37,16 +38,14 @@
 #include "namespaces.h"
 #include "util.h"
 
-#include "gabble-roomlist-channel.h"
-
-#define TP_TYPE_ROOM_STRUCT (dbus_g_type_get_struct ("GValueArray", \
+#define GABBLE_TP_TYPE_ROOM_STRUCT (dbus_g_type_get_struct ("GValueArray", \
       G_TYPE_UINT, \
       G_TYPE_STRING, \
       dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), \
       G_TYPE_INVALID))
 
-#define TP_TYPE_ROOM_LIST (dbus_g_type_get_collection ("GPtrArray", \
-      TP_TYPE_ROOM_STRUCT))
+#define GABBLE_TP_TYPE_ROOM_LIST (dbus_g_type_get_collection ("GPtrArray", \
+      GABBLE_TP_TYPE_ROOM_STRUCT))
 
 static void channel_iface_init (gpointer, gpointer);
 static void roomlist_iface_init (gpointer, gpointer);
@@ -136,7 +135,8 @@ gabble_roomlist_channel_get_property (GObject    *object,
                                 GParamSpec *pspec)
 {
   GabbleRoomlistChannel *chan = GABBLE_ROOMLIST_CHANNEL (object);
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
+  GabbleRoomlistChannelPrivate *priv =
+    GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
 
   switch (property_id) {
     case PROP_OBJECT_PATH:
@@ -170,8 +170,10 @@ gabble_roomlist_channel_set_property (GObject     *object,
                                 GParamSpec   *pspec)
 {
   GabbleRoomlistChannel *chan = GABBLE_ROOMLIST_CHANNEL (object);
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  GabbleRoomlistChannelPrivate *priv =
+    GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
+  TpBaseConnection *conn;
+  TpHandleRepoIface *room_handles;
   TpHandleSet *new_signalled_rooms;
 
   switch (property_id) {
@@ -191,8 +193,10 @@ gabble_roomlist_channel_set_property (GObject     *object,
       priv->conn = g_value_get_object (value);
       conn = (TpBaseConnection *)priv->conn;
 
-      new_signalled_rooms = tp_handle_set_new (
-          conn->handles[TP_HANDLE_TYPE_ROOM]);
+      room_handles = tp_base_connection_get_handles (conn,
+          TP_HANDLE_TYPE_ROOM);
+
+      new_signalled_rooms = tp_handle_set_new (room_handles);
       if (priv->signalled_rooms != NULL)
         {
           const TpIntSet *add;
@@ -223,7 +227,8 @@ gabble_roomlist_channel_class_init (GabbleRoomlistChannelClass *gabble_roomlist_
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_roomlist_channel_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_roomlist_channel_class, sizeof (GabbleRoomlistChannelPrivate));
+  g_type_class_add_private (gabble_roomlist_channel_class,
+      sizeof (GabbleRoomlistChannelPrivate));
 
   object_class->constructor = gabble_roomlist_channel_constructor;
 
@@ -233,10 +238,14 @@ gabble_roomlist_channel_class_init (GabbleRoomlistChannelClass *gabble_roomlist_
   object_class->dispose = gabble_roomlist_channel_dispose;
   object_class->finalize = gabble_roomlist_channel_finalize;
 
-  g_object_class_override_property (object_class, PROP_OBJECT_PATH, "object-path");
-  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE, "channel-type");
-  g_object_class_override_property (object_class, PROP_HANDLE_TYPE, "handle-type");
-  g_object_class_override_property (object_class, PROP_HANDLE, "handle");
+  g_object_class_override_property (object_class, PROP_OBJECT_PATH,
+      "object-path");
+  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE,
+      "channel-type");
+  g_object_class_override_property (object_class, PROP_HANDLE_TYPE,
+      "handle-type");
+  g_object_class_override_property (object_class, PROP_HANDLE,
+      "handle");
 
   param_spec = g_param_spec_object ("connection", "GabbleConnection object",
                                     "Gabble connection object that owns this "
@@ -263,11 +272,12 @@ gabble_roomlist_channel_class_init (GabbleRoomlistChannelClass *gabble_roomlist_
 
 static void stop_listing (GabbleRoomlistChannel *self);
 
-void
+static void
 gabble_roomlist_channel_dispose (GObject *object)
 {
   GabbleRoomlistChannel *self = GABBLE_ROOMLIST_CHANNEL (object);
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
+  GabbleRoomlistChannelPrivate *priv =
+    GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
 
   if (priv->dispose_has_run)
     return;
@@ -291,11 +301,12 @@ gabble_roomlist_channel_dispose (GObject *object)
     G_OBJECT_CLASS (gabble_roomlist_channel_parent_class)->dispose (object);
 }
 
-void
+static void
 gabble_roomlist_channel_finalize (GObject *object)
 {
   GabbleRoomlistChannel *self = GABBLE_ROOMLIST_CHANNEL (object);
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
+  GabbleRoomlistChannelPrivate *priv =
+    GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
 
   /* free any data held directly by the object here */
 
@@ -328,7 +339,8 @@ static gboolean
 emit_room_signal (gpointer data)
 {
   GabbleRoomlistChannel *chan = data;
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
+  GabbleRoomlistChannelPrivate *priv =
+    GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
 
   if (!priv->listing)
       return FALSE;
@@ -342,7 +354,7 @@ emit_room_signal (gpointer data)
   while (priv->pending_room_signals->len != 0)
     {
       gpointer boxed = g_ptr_array_index (priv->pending_room_signals, 0);
-      g_boxed_free (TP_TYPE_ROOM_STRUCT, boxed);
+      g_boxed_free (GABBLE_TP_TYPE_ROOM_STRUCT, boxed);
       g_ptr_array_remove_index_fast (priv->pending_room_signals, 0);
     }
 
@@ -354,7 +366,7 @@ room_info_cb (gpointer pipeline, GabbleDiscoItem *item, gpointer user_data)
 {
   GabbleRoomlistChannel *chan = user_data;
   GabbleRoomlistChannelPrivate *priv;
-  TpBaseConnection *conn;
+  TpHandleRepoIface *room_handles;
   const char *jid, *category, *type, *var, *name;
   TpHandle handle;
   GHashTable *keys;
@@ -372,7 +384,8 @@ room_info_cb (gpointer pipeline, GabbleDiscoItem *item, gpointer user_data)
 
   g_assert (GABBLE_IS_ROOMLIST_CHANNEL (chan));
   priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
-  conn = (TpBaseConnection *)priv->conn;
+  room_handles = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_ROOM);
 
   jid = item->jid;
   name = item->name;
@@ -383,24 +396,36 @@ room_info_cb (gpointer pipeline, GabbleDiscoItem *item, gpointer user_data)
       0 != strcmp (type, "text"))
     return;
 
-  if (!g_hash_table_lookup_extended (item->features, "http://jabber.org/protocol/muc", &k, &v))
+  if (!g_hash_table_lookup_extended (item->features,
+        "http://jabber.org/protocol/muc", &k, &v))
     {
       /* not muc */
       return;
     }
 
-  DEBUG ("got room identity, name=%s, category=%s, type=%s", name, category, type);
+  handle = tp_handle_ensure (room_handles, jid, NULL, NULL);
+  if (handle == 0)
+    {
+      DEBUG ("ignoring listed room with invalid JID '%s'", jid);
+      return;
+    }
+
+  DEBUG ("got room identity, name=%s, category=%s, type=%s", name,
+      category, type);
 
   keys = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
                                 (GDestroyNotify) tp_g_value_slice_free);
 
+  INSERT_KEY (keys, "handle-name", G_TYPE_STRING, string,
+      tp_handle_inspect (room_handles, handle));
   INSERT_KEY (keys, "name", G_TYPE_STRING, string, name);
 
   if (g_hash_table_lookup_extended (item->features, "muc_membersonly", &k, &v))
     INSERT_KEY (keys, "invite-only", G_TYPE_BOOLEAN, boolean, TRUE);
   if (g_hash_table_lookup_extended (item->features, "muc_open", &k, &v))
     INSERT_KEY (keys, "invite-only", G_TYPE_BOOLEAN, boolean, FALSE);
-  if (g_hash_table_lookup_extended (item->features, "muc_passwordprotected", &k, &v))
+  if (g_hash_table_lookup_extended (item->features,
+        "muc_passwordprotected", &k, &v))
     INSERT_KEY (keys, "password", G_TYPE_BOOLEAN, boolean, TRUE);
   if (g_hash_table_lookup_extended (item->features, "muc_unsecure", &k, &v))
     INSERT_KEY (keys, "password", G_TYPE_BOOLEAN, boolean, FALSE);
@@ -418,11 +443,13 @@ room_info_cb (gpointer pipeline, GabbleDiscoItem *item, gpointer user_data)
     INSERT_KEY (keys, "moderated", G_TYPE_BOOLEAN, boolean, TRUE);
   if (g_hash_table_lookup_extended (item->features, "muc_unmoderated", &k, &v))
     INSERT_KEY (keys, "moderated", G_TYPE_BOOLEAN, boolean, FALSE);
-  if (g_hash_table_lookup_extended (item->features, "muc_nonanonymous", &k, &v))
+  if (g_hash_table_lookup_extended (item->features,
+        "muc_nonanonymous", &k, &v))
     INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, FALSE);
   if (g_hash_table_lookup_extended (item->features, "muc_anonymous", &k, &v))
     INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, TRUE);
-  if (g_hash_table_lookup_extended (item->features, "muc_semianonymous", &k, &v))
+  if (g_hash_table_lookup_extended (item->features,
+        "muc_semianonymous", &k, &v))
     INSERT_KEY (keys, "anonymous", G_TYPE_BOOLEAN, boolean, FALSE);
   if (g_hash_table_lookup_extended (item->features, "muc_persistent", &k, &v))
     INSERT_KEY (keys, "persistent", G_TYPE_BOOLEAN, boolean, TRUE);
@@ -442,14 +469,13 @@ room_info_cb (gpointer pipeline, GabbleDiscoItem *item, gpointer user_data)
   if (var != NULL)
     INSERT_KEY (keys, "language", G_TYPE_STRING, string, var);
 
-  handle = gabble_handle_for_room (
-      conn->handles[TP_HANDLE_TYPE_ROOM], jid);
-
+  /* transfer the room handle ref to signalled_rooms */
   tp_handle_set_add (priv->signalled_rooms, handle);
+  tp_handle_unref (room_handles, handle);
 
-  g_value_init (&room, TP_TYPE_ROOM_STRUCT);
+  g_value_init (&room, GABBLE_TP_TYPE_ROOM_STRUCT);
   g_value_take_boxed (&room,
-      dbus_g_type_specialized_construct (TP_TYPE_ROOM_STRUCT));
+      dbus_g_type_specialized_construct (GABBLE_TP_TYPE_ROOM_STRUCT));
 
   dbus_g_type_struct_set (&room,
       0, handle,
@@ -466,7 +492,8 @@ static void
 rooms_end_cb (gpointer data, gpointer user_data)
 {
   GabbleRoomlistChannel *chan = user_data;
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
+  GabbleRoomlistChannelPrivate *priv =
+    GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (chan);
 
   emit_room_signal (chan);
 
@@ -481,7 +508,8 @@ rooms_end_cb (gpointer data, gpointer user_data)
 static void
 stop_listing (GabbleRoomlistChannel *self)
 {
-  GabbleRoomlistChannelPrivate *priv = GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
+  GabbleRoomlistChannelPrivate *priv =
+    GABBLE_ROOMLIST_CHANNEL_GET_PRIVATE (self);
 
   if (priv->listing)
     {
@@ -655,10 +683,12 @@ gabble_roomlist_channel_stop_listing (TpSvcChannelTypeRoomList *iface,
   g_assert (GABBLE_IS_ROOMLIST_CHANNEL (self));
 
   stop_listing (self);
+
+  tp_svc_channel_type_room_list_return_from_stop_listing (context);
 }
 
 static void
-channel_iface_init(gpointer g_iface, gpointer iface_data)
+channel_iface_init (gpointer g_iface, gpointer iface_data)
 {
   TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
 
@@ -672,9 +702,10 @@ channel_iface_init(gpointer g_iface, gpointer iface_data)
 }
 
 static void
-roomlist_iface_init(gpointer g_iface, gpointer iface_data)
+roomlist_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcChannelTypeRoomListClass *klass = (TpSvcChannelTypeRoomListClass *)g_iface;
+  TpSvcChannelTypeRoomListClass *klass =
+    (TpSvcChannelTypeRoomListClass *)g_iface;
 
 #define IMPLEMENT(x) tp_svc_channel_type_room_list_implement_##x (\
     klass, gabble_roomlist_channel_##x)
