@@ -18,6 +18,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
+
+#include <time.h>
+#include <unistd.h>
+
 #include <telepathy-glib/debug.h>
 #include <telepathy-glib/run.h>
 #include "debug.h"
@@ -30,6 +35,27 @@ construct_cm (void)
       GABBLE_TYPE_CONNECTION_MANAGER, NULL);
 }
 
+#ifdef ENABLE_DEBUG
+static void
+stamp_log (const gchar *log_domain,
+           GLogLevelFlags log_level,
+           const gchar *message,
+           gpointer user_data)
+{
+  GTimeVal now;
+  gchar now_str[32];
+  gchar *tmp;
+  struct tm tm;
+
+  g_get_current_time (&now);
+  localtime_r (&(now.tv_sec), &tm);
+  strftime (now_str, 32, "%Y-%m-%d %H:%M:%S", &tm);
+  tmp = g_strdup_printf ("%s.%06ld: %s", now_str, now.tv_usec, message);
+  g_log_default_handler (log_domain, log_level, tmp, NULL);
+  g_free (tmp);
+}
+#endif
+
 int
 main (int argc,
       char **argv)
@@ -39,11 +65,18 @@ main (int argc,
 #ifdef ENABLE_DEBUG
   gabble_debug_set_flags_from_env ();
 
-  /* backwards compatibility */
+  if (g_getenv ("GABBLE_TIMING"))
+    g_log_set_handler (NULL, G_LOG_LEVEL_DEBUG, stamp_log, NULL);
+
   if (g_getenv ("GABBLE_PERSIST"))
     {
-      gabble_debug_set_flags (0xffff);
-      tp_debug_set_all_flags ();
+#ifdef HAVE_TP_DEBUG_SET_FLAGS
+      /* tp-glib >= 0.6.1: persist is no longer a flag in quite the same way */
+      tp_debug_set_persistent (TRUE);
+#else
+      /* tp-glib < 0.6.1: persist is a flag, of sorts */
+      tp_debug_set_flags_from_string ("persist");
+#endif
     }
 #endif
 
