@@ -575,9 +575,7 @@ _grab_nickname (GabblePresenceCache *cache,
 
   if (tp_strdiff (presence->nickname, nickname))
     {
-      if (NULL != presence->nickname)
-        g_free (presence->nickname);
-
+      g_free (presence->nickname);
       presence->nickname = g_strdup (nickname);
       g_signal_emit (cache, signals[NICKNAME_UPDATE], 0, handle);
     }
@@ -814,6 +812,14 @@ _caps_disco_cb (GabbleDisco *disco,
         caps |= PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO;
       else if (0 == strcmp (var, NS_CHAT_STATES))
         caps |= PRESENCE_CAP_CHAT_STATES;
+      else if (0 == strcmp (var, NS_SI))
+        caps |= PRESENCE_CAP_SI;
+      else if (0 == strcmp (var, NS_BYTESTREAMS))
+        caps |= PRESENCE_CAP_BYTESTREAMS;
+      else if (0 == strcmp (var, NS_IBB))
+        caps |= PRESENCE_CAP_IBB;
+      else if (0 == strcmp (var, NS_TUBES))
+        caps |= PRESENCE_CAP_SI_TUBES;
     }
 
   caps |= _detect_h263_n800_hack (node);
@@ -938,11 +944,8 @@ _process_caps_uri (GabblePresenceCache *cache,
 
       if (presence)
         {
-          GabblePresenceCapabilities save_caps = presence->caps;
           gabble_presence_set_capabilities (presence, resource, info->caps,
               serial);
-          g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
-              handle, save_caps, presence->caps);
           DEBUG ("caps for %d (%s) now %d", handle, from, presence->caps);
         }
       else
@@ -1003,6 +1006,7 @@ _process_caps_uri (GabblePresenceCache *cache,
 
 static void
 _process_caps (GabblePresenceCache *cache,
+               GabblePresence *presence,
                TpHandle handle,
                const gchar *from,
                LmMessageNode *lm_node)
@@ -1010,6 +1014,7 @@ _process_caps (GabblePresenceCache *cache,
   const gchar *resource;
   GSList *uris, *i;
   GabblePresenceCachePrivate *priv;
+  GabblePresenceCapabilities old_caps = 0;
   guint serial;
 
   priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
@@ -1021,11 +1026,32 @@ _process_caps (GabblePresenceCache *cache,
 
   uris = _extract_cap_bundles (lm_node);
 
+  if (presence)
+      old_caps = presence->caps;
+
   for (i = uris; NULL != i; i = i->next)
     {
-      _process_caps_uri (cache, from, (gchar *) i->data, handle, resource,
-          serial);
+      _process_caps_uri (cache, from, (gchar *) i->data, handle, resource, serial);
       g_free (i->data);
+
+    }
+
+  if (presence && (old_caps != presence->caps))
+    {
+      DEBUG ("Emitting caps update: handle %u, old %u, new %u",
+          handle, old_caps, presence->caps);
+
+      g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
+          handle, old_caps, presence->caps);
+    }
+  else if (!presence)
+    {
+      DEBUG ("No presence, not updating caps for %u", handle);
+    }
+  else
+    {
+      DEBUG ("No change in caps %u for handle %u, not updating",
+          presence->caps, handle);
     }
 
   g_slist_free (uris);
@@ -1081,9 +1107,12 @@ _parse_presence_message (GabblePresenceCache *cache,
       gabble_presence_cache_update (cache, handle, resource, presence_id,
           status_message, priority);
 
+      if (!presence)
+          presence = gabble_presence_cache_get (cache, handle);
+
       _grab_nickname (cache, handle, from, presence_node);
       _grab_avatar_sha1 (cache, handle, from, presence_node);
-      _process_caps (cache, handle, from, presence_node);
+      _process_caps (cache, presence, handle, from, presence_node);
 
       ret = LM_HANDLER_RESULT_REMOVE_MESSAGE;
       break;
@@ -1310,31 +1339,3 @@ void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
   info->trust = CAPABILITY_BUNDLE_ENOUGH_TRUST;
   info->caps |= new_caps;
 }
-
-gboolean
-gabble_presence_cache_pep_nick_event_handler (GabbleConnection *conn,
-                                              LmMessage *msg,
-                                              TpHandle handle)
-{
-  const gchar *from;
-  LmMessageNode *node;
-
-  from = lm_message_node_get_attribute (msg->node, "from");
-  if (NULL == from)
-    {
-      NODE_DEBUG (msg->node, "PEP event without from attribute, ignoring");
-      return FALSE;
-    }
-
-  node = lm_message_node_find_child (msg->node, "item");
-  if (NULL == node)
-    {
-    NODE_DEBUG (msg->node, "PEP event without item node, ignoring");
-    return FALSE;
-  }
-
-  _grab_nickname (conn->presence_cache, handle, from, node);
-
-  return TRUE;
-}
-
