@@ -22,6 +22,7 @@
 
 #include <string.h>
 #include <glib.h>
+#include <time.h>
 
 #include "presence-cache.h"
 #include "namespaces.h"
@@ -44,6 +45,7 @@ struct _Resource {
     GabblePresenceId status;
     gchar *status_message;
     gint8 priority;
+    time_t last_activity;
 };
 
 typedef struct _GabblePresencePrivate GabblePresencePrivate;
@@ -63,6 +65,7 @@ _resource_new (gchar *name)
   new->status_message = NULL;
   new->priority = 0;
   new->caps_serial = 0;
+  new->last_activity = 0;
 
   return new;
 }
@@ -113,6 +116,28 @@ gabble_presence_new (void)
   return g_object_new (GABBLE_TYPE_PRESENCE, NULL);
 }
 
+static gboolean
+resource_better_than (Resource *a, Resource *b)
+{
+    if (a->priority < 0)
+        return FALSE;
+
+    if (NULL == b)
+        return TRUE;
+
+    if (a->status < b->status)
+        return FALSE;
+    else if (a->status > b->status)
+        return TRUE;
+
+    if (a->last_activity < b->last_activity)
+        return FALSE;
+    else if (a->last_activity > b->last_activity)
+        return TRUE;
+
+    return (a->priority > b->priority);
+}
+
 const gchar *
 gabble_presence_pick_resource_by_caps (
     GabblePresence *presence,
@@ -126,10 +151,9 @@ gabble_presence_pick_resource_by_caps (
     {
       Resource *res = (Resource *) i->data;
 
-      if ((res->priority >= 0) &&
-          ((res->caps & caps) == caps) &&
-          (NULL == chosen || res->priority > chosen->priority))
-        chosen = res;
+      if (((res->caps & caps) == caps) &&
+          (resource_better_than (res, chosen)))
+              chosen = res;
     }
 
   if (chosen)
@@ -302,6 +326,9 @@ gabble_presence_update (GabblePresence *presence,
         }
 
       res->priority = priority;
+
+      if (res->status >= GABBLE_PRESENCE_AVAILABLE)
+          res->last_activity = time (NULL);
     }
 
   /* select the most preferable Resource and update presence->* based on our
