@@ -31,6 +31,7 @@
 
 #include <telepathy-glib/handle-repo-dynamic.h>
 
+#include "conn-aliasing.h"
 #include "sha1/sha1.h"
 #include "namespaces.h"
 #include "gabble-connection.h"
@@ -59,6 +60,18 @@ sha1_hex (const gchar *bytes, guint len)
   return hex_hash;
 }
 
+void
+sha1_bin (const gchar *bytes,
+          guint len,
+          guchar out[SHA1_HASH_SIZE])
+{
+  SHA1Context sc;
+
+  SHA1Init (&sc);
+  SHA1Update (&sc, bytes, len);
+  SHA1Final (&sc, (uint8_t *) out);
+}
+
 static void
 lm_message_node_add_nick (LmMessageNode *node, const gchar *nick)
 {
@@ -180,6 +193,51 @@ lm_message_node_get_child_with_namespace (LmMessageNode *node,
   return NULL;
 }
 
+static LmMessageNode *
+message_node_last_child (LmMessageNode *node)
+{
+  LmMessageNode *l;
+
+  g_return_val_if_fail (node != NULL, NULL);
+
+  if (!node->children)
+    {
+      return NULL;
+    }
+
+  l = node->children;
+
+  while (l->next)
+    {
+      l = l->next;
+    }
+
+  return l;
+}
+
+void
+lm_message_node_add_child_node (LmMessageNode *node, LmMessageNode *child)
+{
+  LmMessageNode *prev;
+
+  g_return_if_fail (node != NULL);
+
+  prev = message_node_last_child (node);
+  lm_message_node_ref (child);
+
+  if (prev)
+    {
+      prev->next = child;
+      child->prev = prev;
+    }
+  else
+    {
+      node->children = child;
+    }
+
+  child->parent = node;
+}
+
 /* note: these are only used internally for readability, not part of the API
  */
 enum {
@@ -632,14 +690,37 @@ lm_message_node_extract_properties (LmMessageNode *node,
         {
           gvalue = g_slice_new0 (GValue);
           g_value_init (gvalue, G_TYPE_INT);
-          g_value_set_int (gvalue, atoi (value));
+          g_value_set_int (gvalue, strtol (value, NULL, 10));
           g_hash_table_insert (properties, g_strdup (name), gvalue);
         }
       else if (0 == strcmp (type, "uint"))
         {
           gvalue = g_slice_new0 (GValue);
           g_value_init (gvalue, G_TYPE_UINT);
-          g_value_set_uint (gvalue, atoi (value));
+          g_value_set_uint (gvalue, strtoul (value, NULL, 10));
+          g_hash_table_insert (properties, g_strdup (name), gvalue);
+        }
+      else if (0 == strcmp (type, "bool"))
+        {
+          gboolean val;
+
+          if (!tp_strdiff (value, "0") || !tp_strdiff (value, "false"))
+            {
+              val = FALSE;
+            }
+          else if (!tp_strdiff (value, "1") || !tp_strdiff (value, "true"))
+            {
+              val = TRUE;
+            }
+          else
+            {
+              g_debug ("invalid boolean value: %s", value);
+              continue;
+            }
+
+          gvalue = g_slice_new0 (GValue);
+          g_value_init (gvalue, G_TYPE_BOOLEAN);
+          g_value_set_boolean (gvalue, val);
           g_hash_table_insert (properties, g_strdup (name), gvalue);
         }
     }
@@ -680,6 +761,10 @@ set_child_from_property (gpointer key,
     {
       type = "uint";
     }
+  else if (G_VALUE_TYPE (gvalue) == G_TYPE_BOOLEAN)
+    {
+      type = "bool";
+    }
   else
     {
       /* a type we don't know how to handle: ignore it */
@@ -702,7 +787,7 @@ set_child_from_property (gpointer key,
 
       type = "bytes";
       arr = g_value_get_boxed (gvalue);
-      str = base64_encode (arr->len, arr->data);
+      str = base64_encode (arr->len, arr->data, FALSE);
       lm_message_node_set_value (child, str);
 
       g_free (str);
@@ -720,11 +805,19 @@ set_child_from_property (gpointer key,
     {
       gchar *str;
 
-      str = g_strdup_printf ("%d", g_value_get_uint (gvalue));
+      str = g_strdup_printf ("%u", g_value_get_uint (gvalue));
       lm_message_node_set_value (child, str);
 
       g_free (str);
     }
+  else if (G_VALUE_TYPE (gvalue) == G_TYPE_BOOLEAN)
+    {
+      /* we output as "0" or "1" despite the canonical representation for
+       * xs:boolean being "false" or "true", for compatibility with older
+       * Gabble versions (OLPC Trial-3) */
+      lm_message_node_set_value (child,
+          g_value_get_boolean (gvalue) ? "1" : "0");
+    }
   else
     {
       g_assert_not_reached ();
