@@ -28,8 +28,10 @@
 #include <loudmouth/loudmouth.h>
 
 #include "gabble-types.h"
+#include "sha1/sha1.h"
 
 gchar *sha1_hex (const gchar *bytes, guint len);
+void sha1_bin (const gchar *bytes, guint len, guchar out[SHA1_HASH_SIZE]);
 void lm_message_node_add_own_nick (LmMessageNode *node,
     GabbleConnection *conn);
 void lm_message_node_unlink (LmMessageNode *orphan);
@@ -45,6 +47,8 @@ G_GNUC_NULL_TERMINATED LmMessage * lm_message_build_with_sub_type (
     const gchar *to, LmMessageType type, LmMessageSubType sub_type,
     guint spec, ...);
 
+void lm_message_node_add_child_node (LmMessageNode *node,
+    LmMessageNode *child);
 
 /* format: a@b/c */
 void gabble_decode_jid (const gchar *jid, gchar **a, gchar **b, gchar **c);
